package com.gree.mall.manager.logic.admin;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.excel.util.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.admin.AdminWebsitTree;
import com.gree.mall.manager.enums.admin.AdminWebsitTypeEnum;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.service.*;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AdminWebsitLogic {

//    @Autowired
//    WorkWechatLogic workWechatLogic;
    @Autowired
    AdminWebsitService adminWebsitService;
    @Autowired
    AdminZoneService adminZoneService;
    @Autowired
    AdminUserWebsitRelaService adminUserWebsitRelaService;
    @Autowired
    AdminCompanyWechatService adminCompanyWechatService;
    @Autowired
    CommonLogic commonLogic;
    @Autowired
    StorageService storageService;
//    @Autowired
//    EnterpriseWechatWorkerLogic enterpriseWechatWorkerLogic;

    /**
     * 区域列表
     */
    public List<AdminZone> adminZoneList(){
        List<AdminZone> list = adminZoneService.list();
        return list;
    }

    /**
     * 部门树
     */
    public List<AdminWebsitTree> tree(HttpServletRequest request){
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        List<AdminWebsit> list = adminWebsitService.lambdaQuery()
                .and(CollectionUtils.isNotEmpty(adminUser.getCompanyWechatIds()), v -> v.in(AdminWebsit::getCompanyWechatId,adminUser.getCompanyWechatIds()))
                .list();
        List<AdminWebsitTree> trees = new ArrayList<>();
        for(AdminWebsit adminWebsit : list){
            AdminWebsitTree adminWebsitTree = new AdminWebsitTree();
            BeanUtils.copyProperties(adminWebsit,adminWebsitTree);
            trees.add(adminWebsitTree);
        }
        AdminUserWebsitRela one = adminUserWebsitRelaService.lambdaQuery().eq(AdminUserWebsitRela::getAdminUserId, adminUser.getAdminUserId()).last("limit 1").one();
        List<AdminWebsitTree> parentList = trees.stream()
                .filter(x -> (one == null && x.getParentId().equals("0")) || (one != null && x.getWebsitId().equals(one.getAdminWebsitId()))).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(parentList)){
            parentList = trees.stream().filter(x -> x.getParentId().equals("0")).collect(Collectors.toList());
        }
        for(AdminWebsitTree adminWebsit:parentList){
            adminWebsit.setChildren(this.treeModule(trees,adminWebsit.getWebsitId()));
        }
        return parentList;
    }


    public List<AdminWebsit> list(String type,Boolean isAll,Boolean isIncre,Boolean status){
        AdminUserCom adminUser = commonLogic.getAdminUser();

        List<AdminWebsit> list = adminWebsitService.lambdaQuery()
                .eq(StringUtils.isNotBlank(type),AdminWebsit::getType, type)
                .eq(isIncre != null,AdminWebsit::getIsIncre, isIncre)
                .eq(status != null,AdminWebsit::getStatus, status)
                .in(CollectionUtils.isNotEmpty(adminUser.getAdminWebsitIds()) && (isAll == null || !isAll),AdminWebsit::getWebsitId,adminUser.getAdminWebsitIds())
                .in(CollectionUtils.isNotEmpty(adminUser.getCompanyWechatIds()),AdminWebsit::getCompanyWechatId,adminUser.getCompanyWechatIds())
                .list();
        return list;
    }


    /**
     *  新增部门
     */
    @Transactional
    public void add(AdminWebsit adminWebsit, HttpServletRequest request) throws WxErrorException, RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Integer count = adminWebsitService.lambdaQuery()
                .eq(StringUtils.isNotBlank(adminUser.getCompanyWechatId()),AdminWebsit::getCompanyWechatId,adminUser.getCompanyWechatId())
                .eq(AdminWebsit::getName, adminWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称已存在");
        }
        AdminWebsit parent = adminWebsitService.getById(adminWebsit.getParentId());
        if(parent == null){
            throw new RemoteServiceException("请选择父部门");
        }
        if(StringUtils.isNotBlank(parent.getCompanyWechatId())) {
            //最多3个部门
            final int max = 3;
            Integer count1 = adminWebsitService.lambdaQuery()
                    .eq(AdminWebsit::getCompanyWechatId, parent.getCompanyWechatId())
                    .eq(AdminWebsit::getType, AdminWebsitTypeEnum.C.getKey()).count();
            if(count1 >= max){
                throw new RemoteServiceException("创建网点数量已达上限3，请联系相关人员");
            }
        }

        adminWebsit.setType(AdminWebsitTypeEnum.C.getKey());
        adminWebsit.setCompanyWechatId(parent.getCompanyWechatId());
        adminWebsit.setCompanyWechatName(parent.getCompanyWechatName());
        adminWebsit.setWebsitId(IdWorker.getIdStr());
        adminWebsit.setWebsitNumber(adminWebsit.getWebsitId());
        adminWebsit.insert();

        reloadJoinCode(adminWebsit.getCompanyWechatId(),adminWebsit.getJoinCode());

        // 初始化辅材配件默认仓
        initMaterialStorage(adminUser, adminWebsit);
    }

    private void initMaterialStorage(AdminUserCom adminUser, AdminWebsit adminWebsit) {
        if (!adminWebsit.getType().equals(AdminWebsitTypeEnum.C.getKey())) {
            return;
        }
        Storage storage = storageService.lambdaQuery()
                .eq(Storage::getCompanyWechatId, adminWebsit.getCompanyWechatId())
                .eq(Storage::getWebsitId, adminWebsit.getWebsitId())
                .eq(Storage::getType, "辅材,配件")
                .eq(Storage::getIsDefault, true)
                .last("limit 1")
                .one();

        if (Objects.isNull(storage)) {
            storage = new Storage();
            storage.setCompanyWechatId(adminWebsit.getCompanyWechatId())
                    .setCompanyName(adminWebsit.getCompanyWechatName())
                    .setWebsitId(adminWebsit.getWebsitId())
                    .setWebsitName(adminWebsit.getName())
                    .setStorageName("辅材,配件默认仓")
                    .setType("辅材,配件")
                    .setStatus(true)
                    .setIsDefault(true)
                    .insert();
        }
    }

    /**
     * 编辑部门
     */
    @Transactional
    public void update(AdminWebsit adminWebsit,HttpServletRequest request) throws RemoteServiceException, WxErrorException {

        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Integer count = adminWebsitService.lambdaQuery()
                .ne(AdminWebsit::getWebsitId,adminWebsit.getWebsitId())
                .eq(StringUtils.isNotBlank(adminUser.getCompanyWechatId()),AdminWebsit::getCompanyWechatId,adminUser.getCompanyWechatId())
                .eq(AdminWebsit::getName, adminWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称或部门编号已存在");
        }
        if(adminWebsit.getParentId().equals("0")){
            throw new RemoteServiceException("请选择父部门");
        }
        AdminWebsit parent = adminWebsitService.getById(adminWebsit.getParentId());
        if(parent != null && parent.getType().equals(AdminWebsitTypeEnum.B.getKey())){
            adminWebsit.setType(AdminWebsitTypeEnum.C.getKey());
        }
        if(StringUtils.equals(adminWebsit.getWebsitId(),adminWebsit.getParentId())){
            throw new RemoteServiceException("父级部门不可以是自己");
        }
        adminWebsit.updateById();

        AdminWebsit old = adminWebsitService.getById(adminWebsit.getWebsitId());
        //workWechatLogic.updateOrganization(adminWebsit.getWebsitId(),adminWebsit.getName(),adminWebsit.getParentId(),adminUser.getAdminCompanyWechat());
        if(old.getType().equals(AdminWebsitTypeEnum.B.getKey())){
            adminCompanyWechatService.lambdaUpdate()
                    .set(AdminCompanyWechat::getLinkPhone,adminWebsit.getWebsitPhone())
                    .eq(AdminCompanyWechat::getCompanyWechatId,old.getCompanyWechatId()).update();
        }
        //刷新成员部门名称
//        enterpriseWechatWorkerLogic.reloadEnterpriseWechatName(adminWebsit,request);
        reloadJoinCode(old.getCompanyWechatId(),adminWebsit.getJoinCode());

    }

    /**
     * 删除部门
     */
    @Transactional
    public void delete(String websitId,HttpServletRequest request) throws WxErrorException {
        Integer count = adminWebsitService.lambdaQuery().eq(AdminWebsit::getParentId, websitId).count();
        if(count > 0){
            throw new RemoteServiceException("当前部门存在子级，删除失败");
        }
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        final AdminWebsit adminWebsit = adminWebsitService.getById(websitId);
        // 普通网点连带删除辅材配件仓
        if (adminWebsit.getType().equals(AdminWebsitTypeEnum.C.getKey())) {
            storageService.lambdaUpdate()
                    .eq(Storage::getCompanyWechatId, adminWebsit.getCompanyWechatId())
                    .eq(Storage::getWebsitId, adminWebsit.getWebsitId())
                    .remove();
        }

        //递归删除
        List<AdminWebsit> list = new ArrayList<>();
        list.addAll(commonLogic.queryAllChild(list,websitId));
        List<String> ids = list.stream().map(AdminWebsit::getWebsitId).collect(Collectors.toList());
        ids.add(websitId);
        if(ids.size() > 0) {
//            for(String idd : ids){
//                workWechatLogic.deleteOrganization(idd,adminUser.getAdminCompanyWechat());
//            }
            adminWebsitService.lambdaUpdate().in(AdminWebsit::getWebsitId, ids).remove();
        }

        // 删除子级普通网点辅材配件仓
        final List<String> materialWebsitList = list.stream()
                .filter(v -> v.getType().equals(AdminWebsitTypeEnum.C.getKey()))
                .map(AdminWebsit::getWebsitId)
                .collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(materialWebsitList)) {
            storageService.lambdaUpdate()
                    .eq(Storage::getCompanyWechatId, adminWebsit.getCompanyWechatId())
                    .in(Storage::getWebsitId, materialWebsitList)
                    .remove();
        }
    }

    /**
     * 详情
     * @param id
     */
    public AdminWebsit detail(String id){
        AdminWebsit adminWebsit = adminWebsitService.getById(id);
        return adminWebsit;
    }

    /**
     * 刷新之后关联条码
     */
    @Transactional
    public void reloadJoinCode(String companyWechatId,String joinCode){
        if(StringUtils.isBlank(companyWechatId) || joinCode == null){
            return;
        }
        adminWebsitService.lambdaUpdate()
                .set(AdminWebsit::getJoinCode,joinCode)
                .eq(AdminWebsit::getCompanyWechatId,companyWechatId).update();

        adminCompanyWechatService.lambdaUpdate()
                .set(AdminCompanyWechat::getJoinCode,joinCode)
                .eq(AdminCompanyWechat::getCompanyWechatId,companyWechatId).update();

    }



    /**
     * 递归组装数据
     * @param list
     * @param parentId
     * @return
     */
    private List<AdminWebsitTree> treeModule(List<AdminWebsitTree> list, String parentId){
        //最父级资源树
        if(parentId == null || parentId.equals("0")) {
            return null;
        }
        List<AdminWebsitTree> collect = list.stream()
                .filter(e -> e.getParentId().equals(parentId)).collect(Collectors.toList());
        if(collect.size() == 0){
            return null;
        }
        for(AdminWebsitTree bean : collect){
            bean.setChildren(this.treeModule(list, bean.getWebsitId()));
        }
        return collect;
    }





}
